/***************************************************************************
                          katalogdoc.cpp  -  description
                             -------------------
    begin                : Tue Mar 18 14:57:05 CET 2003
    copyright            : (C) 2003 by Stefano Salvador
    email                : salva_ste@tin.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qwidget.h>
#include <qdom.h>

// include files for KDE
#include <klocale.h>
#include <klistview.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <kfilterdev.h>

// application specific includes
#include "katalogdoc.h"
#include "katalog.h"
#include "katalogview.h"
#include "cataloghandler.h"
#include "categoryhandler.h"
#include "cattreeitem.h"

QList<KatalogView> *KatalogDoc::pViewList = 0L;

KatalogDoc::KatalogDoc(QWidget *parent, const char *name) : QObject(parent, name)
{
  if(!pViewList)
  {
    pViewList = new QList<KatalogView>();
  }

  pViewList->setAutoDelete(true);

  logHand = new CatalogHandler();
  oryHand = new CategoryHandler();
}

KatalogDoc::~KatalogDoc()
{
  delete logHand;
  delete oryHand;
}

void KatalogDoc::addView(KatalogView *view)
{
  pViewList->append(view);
}

void KatalogDoc::removeView(KatalogView *view)
{
  pViewList->remove(view);
}
void KatalogDoc::setURL(const KURL &url)
{
  doc_url=url;
}

const KURL& KatalogDoc::URL() const
{
  return doc_url;
}

void KatalogDoc::slotUpdateAllViews(KatalogView *sender)
{
  KatalogView *w;
  if(pViewList)
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
      if(w!=sender)
        w->repaint();
    }
  }

}

bool KatalogDoc::saveModified()
{
  bool completed=true;

  if(modified)
  {
    KatalogApp *win=(KatalogApp *) parent();
    int want_save = KMessageBox::warningYesNoCancel(win,
                                         i18n("The current file has been modified.\n"
                                              "Do you want to save it?"),
                                         i18n("Warning"));
    switch(want_save)
    {
      case KMessageBox::Yes:
           if (doc_url.fileName() == i18n("Untitled"))
           {
             win->slotFileSaveAs();
           }
           else
           {
             saveDocument(URL());
       	   };

       	   deleteContents();
           completed=true;
           break;

      case KMessageBox::No:
           setModified(false);
           deleteContents();
           completed=true;
           break;

      case KMessageBox::Cancel:
           completed=false;
           break;

      default:
           completed=false;
           break;
    }
  }

  return completed;
}

void KatalogDoc::closeDocument()
{
  if( saveModified() )
    deleteContents();
}

bool KatalogDoc::newDocument()
{
  deleteContents();

  modified=false;
  doc_url.setFileName(i18n("Untitled"));

  return true;
}

bool KatalogDoc::openDocument(const KURL& url, const char *format /*=0*/)
{
  if( url.isEmpty() )
    return;
  QString tmpfile;
  KIO::NetAccess::download( url, tmpfile );
  QIODevice * dev = KFilterDev::deviceForFile( tmpfile, "application/x-bzip2" );
  dev->open( IO_ReadOnly );

  // Parse XML file
  QDomDocument domTree;
  int Line;
  int Col;
  QString str;
  if ( !domTree.setContent( dev, true, &str, &Line, &Col ) ) {
    dev->close();
    KIO::NetAccess::removeTempFile( tmpfile );
    return false;
  }

  // create the list of the categories
  QDomElement root = domTree.documentElement();
  QDomNode node;
  node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "CATEGORIES" ) {
      getCategories( node.toElement() );
      break;
    }
    node = node.nextSibling();
  }
  // create the tree of the catalogues
  root = domTree.documentElement();
  node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "CATALOG" ) {
      QDomElement catalog = node.toElement();
      QString name = catalog.attribute( "name" );
      QString desc = catalog.attribute( "description" );
      QDate *date = new QDate( node.toElement().attribute( "date_y" ).toInt(),
                              node.toElement().attribute( "date_m" ).toInt(),
                              node.toElement().attribute( "date_d" ).toInt() );
      QStringList *catl = new QStringList(
        QStringList::split( "::", catalog.attribute( "catList" ) ) );
      CatTreeItem *item = logHand->newCatalog( name, desc, date, catl );
      buildTree( item, catalog );
    }
    node = node.nextSibling();
  }

  dev->close();
  KIO::NetAccess::removeTempFile( tmpfile );
  emit( docChanged() );
  modified=false;
  return true;
}

bool KatalogDoc::saveDocument(const KURL& url, const char *format /*=0*/)
{
  QString tmpfile ( "/tmp/katalog_temp" );
  QIODevice * dev = KFilterDev::deviceForFile( tmpfile, "application/x-bzip2" );
  dev->open( IO_WriteOnly );
  QTextStream *ds = new QTextStream( dev );

  *ds << "<?xml version=\"1.0\" standalone=\"yes\" ?>\n";
  *ds << "<KATALOG>\n";

  *ds << "<CATEGORIES>\n";
  for( QValueList<Category>::iterator it = oryHand->begin(); it != oryHand->end(); ++it )
  {
    *ds << "\t<CATEGORY ";
    *ds << "name=\""         << (*it).Name()         << "\" ";
    *ds << "description=\""  << (*it).Description() << "\" />\n";
  }
  *ds <<  "</CATEGORIES>\n";

  for( QValueList<CatTreeItem *>::iterator it = logHand->logList.begin();
    it != logHand->logList.end(); ++it )
  {
    *ds << "<CATALOG ";
    *ds << "name=\""        << (*it)->entry->Name()                  << "\" ";
    *ds << "description=\"" << (*it)->entry->Description()           << "\" ";
    *ds << "date_d=\""      << (*it)->entry->Date()->day()           << "\" ";
    *ds << "date_m=\""      << (*it)->entry->Date()->month()         << "\" ";
    *ds << "date_y=\""      << (*it)->entry->Date()->year()          << "\" ";
    if( (*it)->entry->CatList() )
      *ds << "catList=\""   << (*it)->entry->CatList()->join( "::" ) << "\">\n";
    else
      *ds << "catList=\"\">\n";
    writeChildren( "\t", ds, (*it)->child );
    *ds << "</CATALOG>\n";
  }

  *ds << "</KATALOG>";

  dev->close();

  KIO::NetAccess::upload( tmpfile, url );

  KIO::NetAccess::removeTempFile( tmpfile );

  delete ds;

  modified=false;
  return true;
}

void KatalogDoc::deleteContents()
{
  logHand->clear();
  oryHand->clear();
  emit( docChanged() );
}

void KatalogDoc::writeChildren( QString indent, QTextStream *ds, CatTreeItem *ct )
{
  CatTreeItem *item = ct;
  while( item )
  {
    *ds << indent << "<ITEM ";
    *ds << "name=\""        << item->entry->Name()                  << "\" ";
    *ds << "description=\"" << item->entry->Description()           << "\" ";
    *ds << "icon=\""        << item->entry->Icon()                  << "\" ";
    *ds << "size=\""        << item->entry->Size()                  << "\" ";
    *ds << "date_d=\""      << item->entry->Date()->day()           << "\" ";
    *ds << "date_m=\""      << item->entry->Date()->month()         << "\" ";
    *ds << "date_y=\""      << item->entry->Date()->year()          << "\" ";
    if( item->entry->CatList() )
      *ds << "catList=\""   << item->entry->CatList()->join( "::" ) << "\" ";
    else
      *ds << "catList=\"\" ";
    *ds << "type=\""        << item->entry->EntryType()             << "\">\n";
    if( item->child )
      writeChildren( indent + "\t", ds, item->child );
    *ds << indent << "</ITEM>\n";
    item = item->next;
  }
}

void KatalogDoc::buildTree( CatTreeItem *parentItem, const QDomElement &parentElement )
{
  CatTreeItem *thisItem;
  QDomNode node = parentElement.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "ITEM" ) {
      QString name = node.toElement().attribute( "name" );
      QString desc = node.toElement().attribute( "desc" );
      QString icon = node.toElement().attribute( "icon" );
      int size = node.toElement().attribute( "size" ).toInt();
      int type = node.toElement().attribute( "type" ).toInt();
      QDate *date = new QDate( node.toElement().attribute( "date_y" ).toInt(),
                              node.toElement().attribute( "date_m" ).toInt(),
                              node.toElement().attribute( "date_d" ).toInt() );
      QStringList *catl = new QStringList(
        QStringList::split( "::", node.toElement().attribute( "catList" ) ) );

      thisItem = logHand->addEntry( parentItem, name, size, desc, date, catl, type );
      thisItem->entry->SetIcon( icon );
      // recursive build of the tree
      buildTree( thisItem, node.toElement() );
    }
  node = node.nextSibling();
  }
}

void KatalogDoc::getCategories( QDomElement parentElement )
{
  QDomNode node = parentElement.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "CATEGORY" ) {
      oryHand->addCategory( node.toElement().attribute( "name" ),
                            node.toElement().attribute( "description" ) );
    }
  node = node.nextSibling();
  }
}
